#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib/gi18n.h>

#ifdef _WIN32
#  include <gdk/gdkwin32.h>
#endif

#include "stardict.h"
#include "conf.h"
#include "utils.h"

#include "prefsdlg.h"

#define LOGO										0
#define	DICTIONARY_SCAN_SETTINGS 					1
#define	DICTIONARY_FONT_SETTINGS 					2
#define	DICTIONARY_SOUND_SETTINGS 					3
#define	MAINWIN_OPTIONS_SETTINGS 					4
#define MAINWIN_SEARCH_WEBSITE_SETTINGS				5
#define NOTIFICATION_AREA_ICON_OPITIONS_SETTINGS	6
#define FLOATWIN_OPTIONS_SETTINGS					7
#define FLOATWIN_SIZE_SETTINGS						8

enum
{
	CATEGORY_COLUMN = 0,
	PAGE_NUM_COLUMN,
	NUM_COLUMNS
};

typedef struct _CategoriesTreeItem	CategoriesTreeItem;

struct _CategoriesTreeItem
{
	gchar			*category;
	
	CategoriesTreeItem 	*children;

	gint			notebook_page;
};

static CategoriesTreeItem dictionary_behavior [] =
{
	{N_("Scan Selection"), NULL, DICTIONARY_SCAN_SETTINGS},
	{N_("Font"), NULL, DICTIONARY_FONT_SETTINGS},
	{N_("Sound"), NULL, DICTIONARY_SOUND_SETTINGS},
	
	{ NULL }
};

static CategoriesTreeItem mainwin_behavior [] =
{
	{N_("Options"), NULL, MAINWIN_OPTIONS_SETTINGS},
	{N_("Search website"), NULL, MAINWIN_SEARCH_WEBSITE_SETTINGS},
	
	{ NULL }
};

static CategoriesTreeItem NotificationAreaIcon_behavior [] =
{
	{N_("Options"), NULL, NOTIFICATION_AREA_ICON_OPITIONS_SETTINGS},
	
	{ NULL }
};

static CategoriesTreeItem floatwin_behavior [] =
{
	{N_("Options"), NULL, FLOATWIN_OPTIONS_SETTINGS},
	{N_("Size"), NULL, FLOATWIN_SIZE_SETTINGS},
	
	{ NULL }
};

static CategoriesTreeItem toplevel [] =
{
	{N_("Dictionary"), dictionary_behavior, LOGO},
	
	{N_("Main window"), mainwin_behavior, LOGO},
	
	{N_("Notification area icon"), NotificationAreaIcon_behavior, LOGO},
	
	{N_("Floating window"), floatwin_behavior, LOGO},

	{ NULL }
};

static gint last_selected_page_num = DICTIONARY_SCAN_SETTINGS;

void PrefsDlg::response_handler (GtkDialog *dialog, gint res_id, PrefsDlg *oPrefsDlg)
{
	switch (res_id) {
		case GTK_RESPONSE_HELP:
    show_help("stardict-prefs");
			break;
		default:
			oPrefsDlg->Close();
	}
}

GtkTreeModel* PrefsDlg::create_categories_tree_model ()
{
	GtkTreeStore *model;
	GtkTreeIter iter;
	CategoriesTreeItem *category = toplevel;

	model = gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	while (category->category) {
		CategoriesTreeItem *sub_category = category->children;		
		gtk_tree_store_append (model, &iter, NULL);
		gtk_tree_store_set (model, &iter, CATEGORY_COLUMN, gettext (category->category), PAGE_NUM_COLUMN, category->notebook_page, -1);

		while (sub_category->category) {
	  		GtkTreeIter child_iter;
	  		gtk_tree_store_append (model, &child_iter, &iter);
			gtk_tree_store_set (model, &child_iter,
				CATEGORY_COLUMN, gettext (sub_category->category),
				PAGE_NUM_COLUMN, sub_category->notebook_page,
			      -1);
	  		sub_category++;
		}      
		category++;
	}
	return GTK_TREE_MODEL (model);	
}

void PrefsDlg::categories_tree_selection_cb (GtkTreeSelection *selection, PrefsDlg *oPrefsDlg)
{
 	GtkTreeIter iter;
	GValue value = {0, };

	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	gtk_tree_model_get_value (oPrefsDlg->categories_tree_model, &iter,
			    PAGE_NUM_COLUMN,
			    &value);

	last_selected_page_num = g_value_get_int (&value);

	if (oPrefsDlg->notebook != NULL)
		gtk_notebook_set_current_page (GTK_NOTEBOOK (oPrefsDlg->notebook), 
					       last_selected_page_num);      	
	g_value_unset (&value);
}

gboolean PrefsDlg::selection_init (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, PrefsDlg *oPrefsDlg)
{
	GValue value = {0, };
	gint page_num;

	gtk_tree_model_get_value (oPrefsDlg->categories_tree_model, iter,
			    PAGE_NUM_COLUMN,
			    &value);

	page_num = g_value_get_int (&value);

	g_value_unset (&value);

	if (page_num == last_selected_page_num)
	{
		GtkTreeSelection *selection;

		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (oPrefsDlg->categories_tree));

		gtk_tree_selection_select_iter (selection, iter);

		gtk_notebook_set_current_page (GTK_NOTEBOOK (oPrefsDlg->notebook), page_num);
    	
		return TRUE;
	}
	return FALSE;
}

void PrefsDlg::categories_tree_realize (GtkWidget *widget, PrefsDlg *oPrefsDlg)
{
	gtk_tree_view_expand_all (GTK_TREE_VIEW (widget));

	gtk_tree_model_foreach (oPrefsDlg->categories_tree_model, 
			(GtkTreeModelForeachFunc) selection_init,
			(gpointer)oPrefsDlg);
}

GtkWidget* PrefsDlg::create_categories_tree ()
{
	GtkWidget *sw;
	GtkTreeModel *model;
	GtkWidget *treeview;
	GtkCellRenderer *renderer;
	GtkTreeSelection *selection;
	GtkTreeViewColumn *column;
 	gint col_offset;
	
	sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
					   GTK_SHADOW_ETCHED_IN);
      	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);
	
	gtk_widget_set_size_request (sw, 140, 240);
	
	model = create_categories_tree_model ();
	
	treeview = gtk_tree_view_new_with_model (model);
	g_object_unref (G_OBJECT (model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	
	categories_tree = treeview;
	categories_tree_model = model;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));

	gtk_tree_selection_set_mode (selection,
				   GTK_SELECTION_SINGLE);

	 /* add column for category */
	renderer = gtk_cell_renderer_text_new ();
  	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  
	col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
							    -1, _("Categories"),
							    renderer, "text",
							    CATEGORY_COLUMN,
							    NULL);
	
  	column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);

	g_signal_connect (selection, "changed", 
			  G_CALLBACK (categories_tree_selection_cb), 
			  this);

	gtk_container_add (GTK_CONTAINER (sw), treeview);

      	g_signal_connect (G_OBJECT (treeview), "realize", 
			  G_CALLBACK (categories_tree_realize), 
			  this);

	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
		
	return sw;
}

void PrefsDlg::setup_logo_page()
{
	GtkWidget *image;
	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.stardict.icon.p[0]);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),image,NULL);
}

void PrefsDlg::on_setup_dictionary_scan_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
	gboolean b = gtk_toggle_button_get_active (button);
	gtk_widget_set_sensitive(oPrefsDlg->scan_modifier_key_vbox,b);
	conf->set_only_scan_while_modifier_key(b);
}

void PrefsDlg::on_setup_dictionary_scan_optionmenu_changed(GtkOptionMenu *option_menu, PrefsDlg *oPrefsDlg)
{	
	gint key = gtk_option_menu_get_history(option_menu);
  conf->set_scan_modifier_key(key);
}

void PrefsDlg::on_setup_dictionary_scan_hide_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
	gboolean hide = gtk_toggle_button_get_active(button);
  conf->set_hide_floatwin_when_modifier_key_released(hide);
}

void PrefsDlg::setup_dictionary_scan_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_CONVERT,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Scan Selection</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	vbox1 = gtk_vbox_new(false, 12);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
	GtkWidget *check_button = gtk_check_button_new_with_mnemonic(_("_Only do scanning while modifier key being pressed."));
	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
	gboolean only_scan_while_modifier_key=conf->get_only_scan_while_modifier_key();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),only_scan_while_modifier_key);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_scan_ckbutton_toggled), this);		
	
	scan_modifier_key_vbox = gtk_vbox_new(false, 6);
	gtk_box_pack_start(GTK_BOX(vbox1), scan_modifier_key_vbox,false,false,0);
	gtk_widget_set_sensitive(scan_modifier_key_vbox,only_scan_while_modifier_key);

	check_button = gtk_check_button_new_with_mnemonic(_("H_ide floating window when modifier key released."));
	gtk_box_pack_start(GTK_BOX(scan_modifier_key_vbox),check_button,false,false,0);	
	gboolean hide_floatwin_when_modifier_key_released=conf->get_hide_floatwin_when_modifier_key_released();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),hide_floatwin_when_modifier_key_released);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_scan_hide_ckbutton_toggled), this);		

	hbox = gtk_hbox_new(false, 12);		
	gtk_box_pack_start(GTK_BOX(scan_modifier_key_vbox), hbox,false,false,0);
	label=gtk_label_new(NULL);
	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Scan modifier _key:"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, .5);		
	GtkWidget *option_menu = gtk_option_menu_new();
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *menuitem;	
#ifndef _WIN32
	menuitem = gtk_menu_item_new_with_mnemonic(_("_Win"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
#endif
	menuitem = gtk_menu_item_new_with_mnemonic(_("_Shift"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	menuitem = gtk_menu_item_new_with_mnemonic(_("_Alt"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	menuitem = gtk_menu_item_new_with_mnemonic(_("_Ctrl"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
	gint scan_modifier_key=conf->get_scan_modifier_key();

	gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), scan_modifier_key);
	
	gtk_label_set_mnemonic_widget(GTK_LABEL(label), option_menu);
	gtk_box_pack_start(GTK_BOX(hbox), option_menu,false,false,0);
	g_signal_connect (G_OBJECT (option_menu), "changed", G_CALLBACK (on_setup_dictionary_scan_optionmenu_changed), this);	
}

void PrefsDlg::on_setup_dictionary_font_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
  gboolean b = gtk_toggle_button_get_active(button);
  gtk_widget_set_sensitive(oPrefsDlg->custom_font_hbox, b);
  conf->set_use_custom_font(b);
}

void PrefsDlg::on_setup_dictionary_font_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
{		
	GtkWidget *dlg = gtk_font_selection_dialog_new(_("Choose dictionary font"));
	gtk_window_set_transient_for (GTK_WINDOW (dlg), GTK_WINDOW (oPrefsDlg->window));
	const gchar *text = gtk_button_get_label(GTK_BUTTON(widget));
	if (strcmp(text,_("Choose")))
		gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dlg), text);
	gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(dlg),_("Dictionary font"));
	gint result = gtk_dialog_run (GTK_DIALOG (dlg));
	gchar *font_name;
	switch (result) {
	case GTK_RESPONSE_OK:
		font_name = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
		if (font_name) {
			gtk_button_set_label(GTK_BUTTON(widget),font_name);
			conf->set_custom_font(font_name);
		}
		break;
		default:
	  /*nothing*/;
	}
	gtk_widget_destroy (dlg);
}

void PrefsDlg::setup_dictionary_font_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Font</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false, 0);
	GtkWidget *check_button = gtk_check_button_new_with_mnemonic(_("_Use custom font."));
	gtk_box_pack_start(GTK_BOX(vbox1),check_button,false,false,0);
	gboolean use_custom_font=conf->get_use_custom_font();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),use_custom_font);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_font_ckbutton_toggled), this);		
	custom_font_hbox = gtk_hbox_new(false, 12);	
	gtk_box_pack_start(GTK_BOX(vbox1),custom_font_hbox,false,false,0);
	gtk_widget_set_sensitive(custom_font_hbox,use_custom_font);
	label=gtk_label_new(NULL);
	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Dictionary _font:"));
	gtk_box_pack_start(GTK_BOX(custom_font_hbox),label,false,false,0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, .5);	
	GtkWidget *button;
	const gchar *custom_font=conf->get_custom_font();

	if (custom_font && custom_font[0])
		button = gtk_button_new_with_label(custom_font);
	else
		button=gtk_button_new_with_label(_("Choose"));	

	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
	gtk_box_pack_start(GTK_BOX(custom_font_hbox),button,false,false,0);
	g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (on_setup_dictionary_font_button_clicked), this);
}

void PrefsDlg::on_setup_dictionary_sound_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
	gboolean enable = gtk_toggle_button_get_active(button);
  conf->set_enable_sound_event(enable);
}

void PrefsDlg::setup_dictionary_sound_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_YES,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Sound</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *hbox1;
	hbox1 = gtk_hbox_new(false,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox1,false,false,0);
	
	GtkWidget *check_button;
	check_button = gtk_check_button_new_with_mnemonic(_("_Enable sound event."));
	gboolean enable=conf->get_enable_sound_event();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),enable);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_dictionary_sound_ckbutton_toggled), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
}

void PrefsDlg::on_setup_mainwin_startup_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
  conf->set_hide_on_startup(gtk_toggle_button_get_active(button));
}

void PrefsDlg::setup_mainwin_options_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_EXECUTE,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Options</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *hbox1;
	hbox1 = gtk_hbox_new(false,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox1,false,false,0);
	
	GtkWidget *check_button;
	check_button = gtk_check_button_new_with_mnemonic(_("Hide main window when _starting StarDict."));
	gboolean hide=conf->get_hide_on_startup();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),hide);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_mainwin_startup_ckbutton_toggled), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
}

void PrefsDlg::write_mainwin_searchwebsite_list()
{
	GtkTreeIter iter;
	gboolean have_iter;
	gchar *website, *website_name, *website_link, *website_searchlink;
	GSList *searchwebsite_list = NULL;
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW (searchwebsite_treeview));
	
	have_iter = gtk_tree_model_get_iter_first(model, &iter);
	while (have_iter) {
		gtk_tree_model_get (model, &iter, 0, &website_name, 1, &website_link, 2, &website_searchlink, -1);
		website = g_strdup_printf("%s\t%s\t%s", website_name, website_link, website_searchlink);
		g_free(website_name);
		g_free(website_link);
		g_free(website_searchlink);
		searchwebsite_list = g_slist_append(searchwebsite_list, website);
		have_iter = gtk_tree_model_iter_next(model, &iter);
	}
	conf->set_search_website_list(searchwebsite_list);
}

void PrefsDlg::on_setup_mainwin_searchwebsite_moveup_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
{
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview));
	GtkTreeModel *model;
	GtkTreeIter iter;
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		GtkTreePath* path = gtk_tree_model_get_path(model, &iter);
		if (gtk_tree_path_prev(path)) {
			GtkTreeIter prev;
			gtk_tree_model_get_iter(model, &prev, path);
			gtk_list_store_swap(GTK_LIST_STORE(model), &iter, &prev);	
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview), path, NULL, false, 0, 0);
			oPrefsDlg->write_mainwin_searchwebsite_list();
		}		
		gtk_tree_path_free(path);		
	}	
}

void PrefsDlg::on_setup_mainwin_searchwebsite_movedown_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
{
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview));
	GtkTreeModel *model;
	GtkTreeIter iter;
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		GtkTreePath* path = gtk_tree_model_get_path(model, &iter);
		gtk_tree_path_next(path);
		GtkTreeIter next;
		if (gtk_tree_model_get_iter(model, &next, path)) {
			gtk_list_store_swap(GTK_LIST_STORE(model), &iter, &next);	
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview), path, NULL, false, 0, 0);
			oPrefsDlg->write_mainwin_searchwebsite_list();
		}		
		gtk_tree_path_free(path);		
	}	
}

void PrefsDlg::searchwebsite_add_dialog_cb (GtkDialog *dialog, gint id, PrefsDlg *oPrefsDlg)
{
	switch (id) {
		case GTK_RESPONSE_OK:
		{
			gchar *error_msg = NULL;
			const gchar *website_name = gtk_entry_get_text (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_name_entry));
			const gchar *website_link = gtk_entry_get_text (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_link_entry));
			const gchar *website_searchlink = gtk_entry_get_text (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_searchlink_entry));
			if (!website_name[0])
				error_msg = _("Please input the website name.");
			else if (!website_link[0])
				error_msg = _("Please input the website link.");
			else if (!website_searchlink[0])
				error_msg = _("Please input the website search link.");
			else if (!strstr(website_searchlink, "%s")) {
				error_msg = _("The website search link should contains a \"%%s\" string for quering word.");
			}
			
			if (error_msg) {
				GtkWidget *message_dlg;

				message_dlg = gtk_message_dialog_new (
					GTK_WINDOW (oPrefsDlg->searchwebsite_add_dialog),
					(GtkDialogFlags) (GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
					GTK_MESSAGE_INFO,
					GTK_BUTTONS_OK,
					error_msg);

				gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);

				gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);

				gtk_dialog_run (GTK_DIALOG (message_dlg));
  				gtk_widget_destroy (message_dlg);
				return;
			}			
			GtkListStore *model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(oPrefsDlg->searchwebsite_treeview)));
			GtkTreeIter iter;
			gtk_list_store_prepend (model, &iter);				
			gtk_list_store_set (model, &iter, 0, website_name, 1, website_link, 2, website_searchlink, 3, true, -1);			
			oPrefsDlg->write_mainwin_searchwebsite_list();
			break;
		}
	}
	gtk_widget_destroy (GTK_WIDGET (dialog));
}

void PrefsDlg::on_setup_mainwin_searchwebsite_add_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
{
	if (oPrefsDlg->searchwebsite_add_dialog == NULL)
    {
    	oPrefsDlg->searchwebsite_add_dialog = gtk_dialog_new_with_buttons (_("Add"),
						GTK_WINDOW (oPrefsDlg->window),
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_CANCEL,
						GTK_STOCK_OK,
						GTK_RESPONSE_OK,
						NULL);
		GtkWidget *table = gtk_table_new(3, 2, false);
		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (oPrefsDlg->searchwebsite_add_dialog)->vbox), table);
		gtk_container_set_border_width (GTK_CONTAINER (table), 6);	
    	GtkWidget *label = gtk_label_new_with_mnemonic ("Website _name:");   
		gtk_misc_set_alignment (GTK_MISC (label), 0, .5);
    	oPrefsDlg->searchwebsite_add_dialog_name_entry = gtk_entry_new ();
   		gtk_entry_set_activates_default (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_name_entry), TRUE);    
		gtk_label_set_mnemonic_widget (GTK_LABEL (label), oPrefsDlg->searchwebsite_add_dialog_name_entry);
		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, (GtkAttachOptions)0, 6, 4);
		gtk_table_attach(GTK_TABLE(table), oPrefsDlg->searchwebsite_add_dialog_name_entry, 1, 2, 0, 1, GTK_FILL, (GtkAttachOptions)0, 6, 4);


    	label = gtk_label_new_with_mnemonic ("Website _link:");    
		gtk_misc_set_alignment (GTK_MISC (label), 0, .5);
    	oPrefsDlg->searchwebsite_add_dialog_link_entry = gtk_entry_new ();
   		gtk_entry_set_activates_default (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_link_entry), TRUE);    
		gtk_label_set_mnemonic_widget (GTK_LABEL (label), oPrefsDlg->searchwebsite_add_dialog_link_entry);
		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, (GtkAttachOptions)0, 6, 4);
		gtk_table_attach(GTK_TABLE(table), oPrefsDlg->searchwebsite_add_dialog_link_entry, 1, 2, 1, 2, GTK_FILL, (GtkAttachOptions)0, 6, 4);

    	label = gtk_label_new_with_mnemonic ("Website _search link:");    
		gtk_misc_set_alignment (GTK_MISC (label), 0, .5);
    	oPrefsDlg->searchwebsite_add_dialog_searchlink_entry = gtk_entry_new ();
   		gtk_entry_set_activates_default (GTK_ENTRY (oPrefsDlg->searchwebsite_add_dialog_searchlink_entry), TRUE);    
		gtk_label_set_mnemonic_widget (GTK_LABEL (label), oPrefsDlg->searchwebsite_add_dialog_searchlink_entry);
		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL, (GtkAttachOptions)0, 6, 4);
		gtk_table_attach(GTK_TABLE(table), oPrefsDlg->searchwebsite_add_dialog_searchlink_entry, 1, 2, 2, 3, GTK_FILL, (GtkAttachOptions)0, 6, 4);
		    
    	gtk_dialog_set_default_response (GTK_DIALOG (oPrefsDlg->searchwebsite_add_dialog), GTK_RESPONSE_OK);
		    
		g_signal_connect (G_OBJECT (oPrefsDlg->searchwebsite_add_dialog), "response", G_CALLBACK (searchwebsite_add_dialog_cb), oPrefsDlg);

		g_signal_connect(G_OBJECT (oPrefsDlg->searchwebsite_add_dialog), "destroy", G_CALLBACK (gtk_widget_destroyed), &(oPrefsDlg->searchwebsite_add_dialog));

		gtk_window_set_resizable (GTK_WINDOW (oPrefsDlg->searchwebsite_add_dialog), FALSE);

		gtk_widget_show_all (GTK_WIDGET (oPrefsDlg->searchwebsite_add_dialog));
    }
    else
		gtk_window_present (GTK_WINDOW (oPrefsDlg->searchwebsite_add_dialog));
}

void PrefsDlg::on_setup_mainwin_searchwebsite_remove_button_clicked(GtkWidget *widget, PrefsDlg *oPrefsDlg)
{
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview));
	GtkTreeModel *model;
	GtkTreeIter iter;
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		if (gtk_list_store_remove(GTK_LIST_STORE(model), &iter)) {
			GtkTreePath* path = gtk_tree_model_get_path(model, &iter);
			gtk_tree_selection_select_path(selection, path);
			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview), path, NULL, false, 0, 0);
			gtk_tree_path_free(path);
		}
		oPrefsDlg->write_mainwin_searchwebsite_list();
	}
}

void PrefsDlg::on_setup_mainwin_searchwebsite_cell_edited(GtkCellRendererText *cell, const gchar *path_string, const gchar *new_text, PrefsDlg *oPrefsDlg)
{
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW (oPrefsDlg->searchwebsite_treeview));
	GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
	GtkTreeIter iter;

	gint column;
	column = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (cell), "column"));
	gtk_tree_model_get_iter (model, &iter, path);

	switch (column) {
	case 0:
	case 1:
		if (new_text[0]) {
			gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, new_text, -1);
			oPrefsDlg->write_mainwin_searchwebsite_list();
		}
		break;
	case 2:
		if (new_text[0]) {
			if (!strstr(new_text, "%s")) {
				GtkWidget *message_dlg;

				message_dlg = gtk_message_dialog_new (
					GTK_WINDOW (oPrefsDlg->window),
					(GtkDialogFlags) (GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
					GTK_MESSAGE_INFO,
					GTK_BUTTONS_OK,
					_("The website search link should contains a \"%%s\" string for quering word."));

				gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);

				gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);

				gtk_dialog_run (GTK_DIALOG (message_dlg));
				gtk_widget_destroy (message_dlg);
			}
			else {
				gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, new_text, -1);
				oPrefsDlg->write_mainwin_searchwebsite_list();
			}
		}
		break;

    }

	gtk_tree_path_free (path);
}

void PrefsDlg::setup_mainwin_searchwebsite_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Search Website</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *vbox2;
	vbox2 = gtk_vbox_new(false, 6);
	gtk_box_pack_start(GTK_BOX(vbox), vbox2, true, true,0);
	
	GtkListStore *model;
	model = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);

	const GSList *searchwebsite_list=conf->get_search_website_list();

	gchar *website;
	gchar *website_link;
	gchar *website_searchlink;
	GtkTreeIter iter;
	const GSList *list = searchwebsite_list;
	while (list) {
		website = (gchar *)(list->data);
		website_link = strchr(website, '\t');
		if (website_link && (website_searchlink = strchr(website_link+1, '\t'))) {			
			*website_link = '\0';
			*website_searchlink = '\0';
			gtk_list_store_append (model, &iter);				
			gtk_list_store_set (model, &iter, 0, website, 1, website_link+1, 2, website_searchlink+1, 3, true, -1);
		}
		list = g_slist_next(list);
	}
	
	GtkWidget *sw;	
	sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
      	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);
	
	gtk_widget_set_size_request (sw, 300, 180);

	searchwebsite_treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL(model));
	g_object_unref (G_OBJECT (model));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (searchwebsite_treeview), TRUE);
	
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (searchwebsite_treeview));

	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);

	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
		
	renderer = gtk_cell_renderer_text_new ();
	g_signal_connect (renderer, "edited", G_CALLBACK (on_setup_mainwin_searchwebsite_cell_edited), this);
  	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);  	
	g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER(0));
	column = gtk_tree_view_column_new_with_attributes (_("Website Name"), renderer, "text", 0, "editable", 3, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(searchwebsite_treeview), column);
  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);

	renderer = gtk_cell_renderer_text_new ();
	g_signal_connect (renderer, "edited", G_CALLBACK (on_setup_mainwin_searchwebsite_cell_edited), this);
  	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);  	
	g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER(1));
	column = gtk_tree_view_column_new_with_attributes (_("Website link"), renderer, "text", 1, "editable", 3, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(searchwebsite_treeview), column);
  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);

	renderer = gtk_cell_renderer_text_new ();
	g_signal_connect (renderer, "edited", G_CALLBACK (on_setup_mainwin_searchwebsite_cell_edited), this);
  	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);  	
	g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER(2));
	column = gtk_tree_view_column_new_with_attributes (_("Website search link"), renderer, "text", 2, "editable", 3, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(searchwebsite_treeview), column);
  	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
	
	gtk_container_add (GTK_CONTAINER (sw), searchwebsite_treeview);
	gtk_box_pack_start (GTK_BOX (vbox2), sw, TRUE, TRUE, 0);

	GtkWidget *hbox1;
	hbox1 = gtk_hbox_new(false,6);
	GtkWidget *button;
	button = gtk_button_new();
	image = gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(button), image);
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_setup_mainwin_searchwebsite_moveup_button_clicked), this);
	gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
	button = gtk_button_new();
	image = gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(button), image);
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_setup_mainwin_searchwebsite_movedown_button_clicked), this);
	gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
	button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_setup_mainwin_searchwebsite_remove_button_clicked), this);
	gtk_box_pack_end (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
	
/*	button = gtk_button_new();	
	GtkWidget *align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);      
	gtk_container_add (GTK_CONTAINER (button), align);
	GtkWidget *hbox2 = gtk_hbox_new (FALSE, 2);
	gtk_container_add (GTK_CONTAINER (align), hbox2);
	label = gtk_label_new(NULL);
	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("_Modify"));
	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);	
	image = gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_BUTTON);
	gtk_box_pack_start (GTK_BOX (hbox2), image, FALSE, FALSE, 0);
	gtk_box_pack_end (GTK_BOX (hbox2), label, FALSE, FALSE, 0);      
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_setup_mainwin_searchwebsite_edit_button_clicked), this);
	gtk_box_pack_end (GTK_BOX (hbox1), button, FALSE, FALSE, 0);*/
	
	button = gtk_button_new_from_stock(GTK_STOCK_ADD);
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_setup_mainwin_searchwebsite_add_button_clicked), this);
	gtk_box_pack_end (GTK_BOX (hbox1), button, FALSE, FALSE, 0);
	
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, false, false, 0);
}

void PrefsDlg::on_setup_NotificationAreaIcon_QueryInFloatWin_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
	gboolean queryin = gtk_toggle_button_get_active(button);
  conf->set_query_in_floatwin(queryin);
}

void PrefsDlg::setup_NotificationAreaIcon_options_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_DND,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Options</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *hbox1;
	hbox1 = gtk_hbox_new(false,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox1,false,false,0);
	
	GtkWidget *check_button;
	check_button = gtk_check_button_new_with_mnemonic(_("_Query in the floating window when middle mouse\nbutton clicked."));
	gboolean query_in_floatwin=conf->get_query_in_floatwin();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),query_in_floatwin);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_NotificationAreaIcon_QueryInFloatWin_ckbutton_toggled), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
}

void PrefsDlg::on_setup_floatwin_pronounce_ckbutton_toggled(GtkToggleButton *button, PrefsDlg *oPrefsDlg)
{
  conf->set_pronounce_when_popup(gtk_toggle_button_get_active(button));
}

void PrefsDlg::setup_floatwin_options_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_DND,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Options</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *hbox1;
	hbox1 = gtk_hbox_new(false,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox1,false,false,0);
	
	GtkWidget *check_button;
	check_button = gtk_check_button_new_with_mnemonic(_("_Pronouce the word when pop up."));
	gboolean pronounce_when_popup=conf->get_pronounce_when_popup();

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),pronounce_when_popup);
	g_signal_connect (G_OBJECT (check_button), "toggled", G_CALLBACK (on_setup_floatwin_pronounce_ckbutton_toggled), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox1),check_button,false,false,0);
}

void PrefsDlg::on_setup_floatwin_size_max_width_spinbutton_changed(GtkSpinButton *button, PrefsDlg *oPrefsDlg)
{
	gint width = gtk_spin_button_get_value_as_int(button);
  conf->set_max_window_width(width);
}

void PrefsDlg::on_setup_floatwin_size_max_height_spinbutton_changed(GtkSpinButton *button, PrefsDlg *oPrefsDlg)
{
	gint height = gtk_spin_button_get_value_as_int(button);
  conf->set_max_window_height(height);
}

void PrefsDlg::setup_floatwin_size_page()
{
	GtkWidget *vbox;
	vbox = gtk_vbox_new(false,12);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,NULL);
	GtkWidget *vbox1;
	vbox1 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox1,false,false,6);
	GtkWidget *hbox;
	hbox = gtk_hbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox1),hbox,false,false,0);
	GtkWidget *image;
	image = gtk_image_new_from_stock(GTK_STOCK_ZOOM_FIT,GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_box_pack_start(GTK_BOX(hbox),image,false,false,0);
	GtkWidget *label;
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\" size=\"x-large\">Max window size</span>"));
	gtk_box_pack_start(GTK_BOX(hbox),label,false,false,0);
	GtkWidget *hseparator;
	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox1),hseparator,false,false,0);
	
	GtkWidget *vbox2;
	vbox2 = gtk_vbox_new(false,6);
	gtk_box_pack_start(GTK_BOX(vbox),vbox2,false,false,0);
	
	gint 
	  max_width=conf->get_max_window_width(),
	  max_height=conf->get_max_window_height();

	GdkScreen *screen = gtk_window_get_screen(GTK_WINDOW(gpAppFrame->oAppCore.window));
	gint screen_width = gdk_screen_get_width(screen);
	gint screen_height = gdk_screen_get_height(screen);


	GtkWidget *hbox1;
	hbox1= gtk_hbox_new(false,6);
	label=gtk_label_new(NULL);
	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Max window _width:"));
	gtk_box_pack_start(GTK_BOX(hbox1),label,false,false,0);
	GtkWidget *spin_button;
	spin_button = gtk_spin_button_new_with_range(MIN_MAX_FLOATWIN_WIDTH,screen_width,1);
	gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin_button);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin_button), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button),max_width);
	g_signal_connect (G_OBJECT (spin_button), "value-changed", G_CALLBACK (on_setup_floatwin_size_max_width_spinbutton_changed), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox1),spin_button,false,false,0);
	label=gtk_label_new(_("(default:320)"));
	gtk_box_pack_start(GTK_BOX(hbox1),label,false,false,0);
	gtk_box_pack_start(GTK_BOX(vbox2),hbox1,false,false,0);

	GtkWidget *hbox2;
	hbox2= gtk_hbox_new(false,6);
	label=gtk_label_new(NULL);
	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("Max window hei_ght:"));
	gtk_box_pack_start(GTK_BOX(hbox2),label,false,false,0);
	spin_button = gtk_spin_button_new_with_range(MIN_MAX_FLOATWIN_HEIGHT,screen_height,1);
	gtk_label_set_mnemonic_widget(GTK_LABEL(label), spin_button);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin_button), GTK_UPDATE_IF_VALID);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button),max_height);
	g_signal_connect (G_OBJECT (spin_button), "value-changed", G_CALLBACK (on_setup_floatwin_size_max_height_spinbutton_changed), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox2),spin_button,false,false,0);
	label=gtk_label_new(_("(default:240)"));
	gtk_box_pack_start(GTK_BOX(hbox2),label,false,false,0);
	gtk_box_pack_start(GTK_BOX(vbox2),hbox2,false,false,0);
}

GtkWidget* PrefsDlg::create_notebook ()
{
	notebook = gtk_notebook_new();
	GtkNotebook *nb = (GtkNotebook *)notebook;
	gtk_notebook_set_show_tabs(nb,false);
	gtk_notebook_set_show_border(nb,false);
	
	setup_logo_page ();
	setup_dictionary_scan_page ();
	setup_dictionary_font_page ();
	setup_dictionary_sound_page ();
	setup_mainwin_options_page ();
	setup_mainwin_searchwebsite_page();
	setup_NotificationAreaIcon_options_page();
	setup_floatwin_options_page ();
	setup_floatwin_size_page ();
	
	gtk_notebook_set_current_page (nb, LOGO);	
	return notebook;
}


PrefsDlg::PrefsDlg()
{
	window = NULL;
	searchwebsite_add_dialog = NULL;
}

void PrefsDlg::Show()
{	
	if (window) {
		gtk_window_present(GTK_WINDOW(window));
		return;
	}	
	
	window = gtk_dialog_new();
	gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (gpAppFrame->oAppCore.window));

	gtk_dialog_add_button (GTK_DIALOG (window),
                             	GTK_STOCK_HELP,
                             	GTK_RESPONSE_HELP);
	
	gtk_dialog_add_button (GTK_DIALOG (window),
                             	GTK_STOCK_CLOSE,
                             	GTK_RESPONSE_CLOSE);
	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
	g_signal_connect(G_OBJECT (window), "response",
			 G_CALLBACK (response_handler), this);
	GtkWidget *hbox;
	hbox = gtk_hbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
	GtkWidget *r;
	r = gtk_vbox_new (FALSE, 6);
	
	GtkWidget *label;
	label = gtk_label_new_with_mnemonic (_("Cat_egories:"));
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);

	GtkWidget *ct;
	ct = create_categories_tree ();
		
	gtk_box_pack_start (GTK_BOX (r), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (r), ct, TRUE, TRUE, 0);

    GtkWidget *l;
	l = create_notebook ();

	gtk_box_pack_start (GTK_BOX (hbox), r, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), hbox,
			true, true, 0);
	
	gtk_label_set_mnemonic_widget (GTK_LABEL (label), categories_tree);

	gtk_widget_show_all (GTK_DIALOG (window)->vbox);	
	
	gtk_window_set_title (GTK_WINDOW (window), _("Preferences"));
	//gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
	gtk_widget_show(window);
}

void PrefsDlg::Close()
{
	if (window) {
		gtk_widget_destroy (window);
		window = NULL;
	}
}
